@@ -52,7 +52,6 @@ extern char  _STR[150], _STR2[250];
 extern stringpart *labelbuf;
 extern objectpair *pushlist;
 
-#define RADFAC 0.0174532925199   /* pi / 180 */
 extern double atan2();
 
 /*------------------------------------------------------------------------*/
@@ -87,7 +86,7 @@ void objectdefaults(objinstptr newinst, objectptr thisobj, int x, int y)
 {
    newinst->position.x = x;
    newinst->position.y = y;
-   newinst->rotation = 1;
+   newinst->rotation = 0;
    newinst->scale = 1.0;
    newinst->thisobject = thisobj;
    newinst->color = DEFAULTCOLOR;
@@ -155,7 +154,7 @@ void starttext(Widget button, u_int mode, caddr_t calldata)
 
 void labeldefaults(labelptr newlabel, u_char dopin, int x, int y)
 {
-   newlabel->rotation = 1;
+   newlabel->rotation = 0;
    newlabel->color = areastruct.color;
    newlabel->scale = areastruct.textscale;
    newlabel->string = (stringpart *)malloc(sizeof(stringpart));
@@ -565,13 +564,18 @@ void rejustify(short mode)
 {
    labelptr curlabel = NULL;
    short    *tsel;
-   static short transjust[] = {15, 13, 12, 7, 5, 4, 11, 9, 8};
+   short jsave;
+   Boolean changed = False;
+   static short transjust[] = {15, 13, 12, 7, 5, 4, 3, 1, 0};
 
    if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
       curlabel = TOLABEL(EDITPART);
       UDrawTLine(curlabel);
       undrawtext(curlabel);
-      curlabel->justify = transjust[mode] | (curlabel->justify & FLIPINV);
+      jsave = curlabel->justify;
+      curlabel->justify = transjust[mode] |
+		(curlabel->justify & (FLIPINV | PINVISIBLE));
+      if (jsave != curlabel->justify) changed = True;
       redrawtext(curlabel);
       UDrawTLine(curlabel);
 
@@ -582,8 +586,11 @@ void rejustify(short mode)
 	      areastruct.selects; tsel++) {
 	 if (SELECTTYPE(tsel) == LABEL) {
 	    curlabel = SELTOLABEL(tsel);
+            jsave = curlabel->justify;
 	    undrawtext(curlabel);
-      	    curlabel->justify = transjust[mode] | (curlabel->justify & FLIPINV);
+      	    curlabel->justify = transjust[mode] |
+		(curlabel->justify & (FLIPINV | PINVISIBLE));
+            if (jsave != curlabel->justify) changed = True;
 	    redrawtext(curlabel);
 	 }
       }
@@ -591,6 +598,8 @@ void rejustify(short mode)
    }
    if (curlabel == NULL)
       Wprintf("No labels chosen to rejustify");
+   else if (changed)
+      incr_changes(objectdata);
 }
 
 /*----------------------------------*/
