@@ -34,7 +34,6 @@
 #define xmat(a)		(((a) - 3300) << 4)
 #define ymat(a)		((3300 - (a)) << 4)
 #define S_OBLIQUE 13	/* position of Symbol-Oblique in font array */
-#define ROT_UNIT (360 / RSTEPS) /* degrees per xcircuit rotational unit */
 
 /*------------------------------------------------------------------------*/
 /* External Variable definitions                                          */
@@ -400,9 +399,8 @@ void loadlgf(int mode)
 		  (*newinst)->params = NULL;
 
 		  if (pvalx2 & 0x01) pvalx2 ^= 0x02;
-                  if (pvalx2 >= 4) (*newinst)->rotation = -((pvalx2 - 4) *
-		     (90 / ROT_UNIT) + 1);
-                  else (*newinst)->rotation = pvalx2 * (90 / ROT_UNIT) + 1;
+                  if (pvalx2 >= 4) (*newinst)->rotation = -(((pvalx2 - 4) * 90) + 1);
+                  else (*newinst)->rotation = (pvalx2 * 90) + 1;
 		  (*newinst)->thisobject = *libobj;
 
 	          /* Add label to "TO" and "FROM" */
