@@ -46,56 +46,6 @@ extern int *appcolors;
 extern short textpos, textend;
 extern objectpair *pushlist;
 
-/*-----------------------*/
-/* Arithmetic functions  */
-/*-----------------------*/
-
-#define RADFAC 0.0174532925199   /* pi / 180 */
-
-/*------------------------------------------------------------------------*/
-/* A sine and cosine lookup table for fast axis rotation 		  */
-/*------------------------------------------------------------------------*/
-
-float sinevals[] = {0, 0.0872, 0.1736, 0.2588, 0.342, 0.4226, 0.5, 0.5736,
-	0.6427, 0.7071, 0.7660, 0.8192, 0.8660, 0.9063, 0.9397, 0.9659,
-	0.9848, 0.9962, 1,
-	-0.0872, -0.1736, -0.2588, -0.342, -0.4226, -0.5, -0.5736, -0.6427,
-	-0.7071, -0.7660, -0.8192, -0.866, -0.9063, -0.9397, -0.9659,
-	-0.9848, -0.9962, -1};
-
-float *cosine[RSTEPS + 1] = {
-     &sinevals[18], &sinevals[18], &sinevals[17], &sinevals[16],
-     &sinevals[15], &sinevals[14], &sinevals[13], &sinevals[12], &sinevals[11],
-     &sinevals[10], &sinevals[9], &sinevals[8], &sinevals[7], &sinevals[6],
-     &sinevals[5], &sinevals[4], &sinevals[3], &sinevals[2], &sinevals[1],
-     &sinevals[0], &sinevals[19], &sinevals[20], &sinevals[21], &sinevals[22],
-     &sinevals[23], &sinevals[24], &sinevals[25], &sinevals[26], &sinevals[27],
-     &sinevals[28], &sinevals[29], &sinevals[30], &sinevals[31], &sinevals[32],
-     &sinevals[33], &sinevals[34], &sinevals[35], &sinevals[36], &sinevals[35],
-     &sinevals[34], &sinevals[33], &sinevals[32], &sinevals[31], &sinevals[30],
-     &sinevals[29], &sinevals[28], &sinevals[27], &sinevals[26], &sinevals[25],
-     &sinevals[24], &sinevals[23], &sinevals[22], &sinevals[21], &sinevals[20],
-     &sinevals[19], &sinevals[0], &sinevals[1], &sinevals[2], &sinevals[3],
-     &sinevals[4], &sinevals[5], &sinevals[6], &sinevals[7], &sinevals[8],
-     &sinevals[9], &sinevals[10], &sinevals[11], &sinevals[12], &sinevals[13],
-     &sinevals[14], &sinevals[15], &sinevals[16], &sinevals[17]};
-float *sine[RSTEPS + 1] =  {
-     &sinevals[0], &sinevals[0], &sinevals[1], &sinevals[2],
-     &sinevals[3], &sinevals[4], &sinevals[5], &sinevals[6], &sinevals[7],
-     &sinevals[8], &sinevals[9], &sinevals[10], &sinevals[11], &sinevals[12],
-     &sinevals[13], &sinevals[14], &sinevals[15], &sinevals[16], &sinevals[17],
-     &sinevals[18], &sinevals[17], &sinevals[16], &sinevals[15], &sinevals[14],
-     &sinevals[13], &sinevals[12], &sinevals[11], &sinevals[10], &sinevals[9],
-     &sinevals[8], &sinevals[7], &sinevals[6], &sinevals[5], &sinevals[4],
-     &sinevals[3], &sinevals[2], &sinevals[1], &sinevals[0], &sinevals[19],
-     &sinevals[20], &sinevals[21], &sinevals[22], &sinevals[23], &sinevals[24],
-     &sinevals[25], &sinevals[26], &sinevals[27], &sinevals[28], &sinevals[29],
-     &sinevals[30], &sinevals[31], &sinevals[32], &sinevals[33], &sinevals[34],
-     &sinevals[35], &sinevals[36], &sinevals[35], &sinevals[34], &sinevals[33],
-     &sinevals[32], &sinevals[31], &sinevals[30], &sinevals[29], &sinevals[28],
-     &sinevals[27], &sinevals[26], &sinevals[25], &sinevals[24], &sinevals[23],
-     &sinevals[22], &sinevals[21], &sinevals[20], &sinevals[19]};
-
 /*------------------------------------------------------------------------*/
 /* find the squared length of a wire (or distance between two points in   */
 /* user space).								  */
@@ -163,9 +113,9 @@ long finddist(XPoint *linept1, XPoint *linept2, XPoint *userpt)
 
 void calcarc(arcptr thearc)
 {
-   short idx, thidx;
+   short idx;
    int sarc;
-   float theta;
+   float theta, delta;
 
    /* assume that angle2 > angle1 always: must be guaranteed by other routines */
 
@@ -173,25 +123,19 @@ void calcarc(arcptr thearc)
    thearc->number = (sarc / 360) + 1;
    if (sarc % 360 != 0) thearc->number++;
 	   
-   thidx = ((int)(thearc->angle1 * RSTEPS) / 360) + 2;
-   if (thidx < 1) thidx += RSTEPS;
-   else if (thidx > RSTEPS) thidx -= RSTEPS;
-
+   delta = RADFAC * ((float)(thearc->angle2 - thearc->angle1) / (thearc->number - 1));
    theta = thearc->angle1 * RADFAC;
-   thearc->points[0].x = (float)thearc->position.x + 
-	   fabs((float)thearc->radius) * cos(theta);
-   thearc->points[0].y = (float)thearc->position.y +
-	   (float)thearc->yaxis * sin(theta);
 
-   for (idx = 1; idx < thearc->number - 1; idx++) {
+   for (idx = 0; idx < thearc->number - 1; idx++) {
       thearc->points[idx].x = (float)thearc->position.x + 
-	   fabs((float)thearc->radius) * (*cosine[thidx]);
+	   fabs((float)thearc->radius) * cos(theta);
       thearc->points[idx].y = (float)thearc->position.y +
-	   (float)thearc->yaxis * (*sine[thidx]);
-      thidx++;
-      if (thidx == RSTEPS + 1) thidx = 1;
+	   (float)thearc->yaxis * sin(theta);
+      theta += delta;
    }
 
+   /* place last point exactly to avoid roundoff error */
+
    theta = thearc->angle2 * RADFAC;
    thearc->points[thearc->number - 1].x = (float)thearc->position.x + 
 	   fabs((float)thearc->radius) * cos(theta);
@@ -617,21 +561,18 @@ void UMakeWCTM(Matrix *ctm)
 
 void UMultCTM(Matrix *ctm, XPoint position, float scale, short rotate)
 {
-   float tmpa, tmpb, tmpd, tmpe, ascale;
+   float tmpa, tmpb, tmpd, tmpe, yscale;
    float mata, matb, matc;
+   double drot = (double)rotate * RADFAC;
 
-   if (rotate < 0) {  /* implies flipped object */
-      rotate = -rotate;
-      scale = -scale;
-   }
-   ascale = abs(scale);
+   yscale = abs(scale);  /* -scale implies flip in x direction only */ 
    
-   tmpa =  scale * (*cosine[rotate]);
-   tmpb = ascale * (*sine[rotate]);
-   tmpd = -scale * (*sine[rotate]);
-   tmpe = ascale * (*cosine[rotate]);
+   tmpa =  scale * cos(drot);
+   tmpb = yscale * sin(drot);
+   tmpd = -scale * sin(drot);
+   tmpe = yscale * cos(drot);
    
-   mata = ctm->a * tmpa + ctm->d * tmpd;
+   mata = ctm->a * tmpa + ctm->d * tmpb;
    matb = ctm->b * tmpa + ctm->e * tmpb;
    matc = ctm->c * tmpa + ctm->f * tmpb + position.x;
 
@@ -644,9 +585,9 @@ void UMultCTM(Matrix *ctm, XPoint position, float scale, short rotate)
    ctm->c = matc;
 }
 
-/*------------------------------------------------------------------------*/
-/* Slanting function x' = x + beta * y, y' = y				  */
-/*------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Slanting function x' = x + beta * y, y' = y				*/
+/*----------------------------------------------------------------------*/
 
 void USlantCTM(Matrix *ctm, float beta)
 {
@@ -654,39 +595,49 @@ void USlantCTM(Matrix *ctm, float beta)
    ctm->e += ctm->d * beta;
 }
 
-/*------------------------------------------------------------------------*/
+#define EPS 1e-9
+/*----------------------------------------------------------------------*/
+/* Transform text to make it right-side up within 90 degrees of page	*/
+/* NOTE:  This is not yet resolved, as xcircuit does not agree with	*/
+/* PostScript in a few cases!						*/
+/*----------------------------------------------------------------------*/
 
 void UPreScaleCTM(Matrix *ctm)
 {
-   if ((ctm->a < 0) || (ctm->a == 0 && ((ctm->b * ctm->d) < 0))) {
+   /* negative X scale (-1, +1) */
+   if ((ctm->a < -EPS) || ((ctm->a < EPS) && (ctm->a > -EPS) &&
+		((ctm->d * ctm->b) < 0))) {
       ctm->a = -ctm->a;
       ctm->d = -ctm->d;
    }
-   if (ctm->e > 0) {
+
+   /* negative Y scale (+1, -1) */
+   if (ctm->e > EPS) {
       ctm->e = -ctm->e;
       ctm->b = -ctm->b;
    }
+
+   /* At 90, 270 degrees need special attention to avoid discrepencies	*/
+   /* with the PostScript output due to roundoff error.  This code	*/
+   /* matches what PostScript produces.					*/
+
 }
 
-/*------------------------------------------------------------------------*/
-/* Adjust justification and CTM as necessary for flip invariance	  */
-/*------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Adjust justification and CTM as necessary for flip invariance	*/
+/*----------------------------------------------------------------------*/
 
 short flipadjust(short justify)
 {
    short tmpjust = justify & (~FLIPINV);
 
    if (justify & FLIPINV) {
-      if ((DCTM)->a < 0) {
+      if (((DCTM)->a < -EPS) || (((DCTM)->a < EPS) && ((DCTM)->a > -EPS) &&
+		(((DCTM)->d * (DCTM)->b) < 0))) {
          if ((tmpjust & (RIGHT | NOTLEFT)) != NOTLEFT)
             tmpjust ^= (RIGHT | NOTLEFT);
       }   
-      else if ((DCTM)->a == 0) {
-	 if (((DCTM)->b * (DCTM)->d) < 0)
-            if ((tmpjust & (RIGHT | NOTLEFT)) != NOTLEFT)
-               tmpjust ^= (RIGHT | NOTLEFT);
-      }
-      if ((DCTM)->e > 0) {
+      if ((DCTM)->e > EPS) {
          if ((tmpjust & (TOP | NOTBOTTOM)) != NOTBOTTOM) 
             tmpjust ^= (TOP | NOTBOTTOM);
       }
@@ -699,19 +650,16 @@ short flipadjust(short justify)
 
 void UPreMultCTM(Matrix *ctm, XPoint position, float scale, short rotate)
 {
-   float tmpa, tmpb, tmpd, tmpe, ascale;
+   float tmpa, tmpb, tmpd, tmpe, yscale;
    float mata, matd;
+   double drot = (double)rotate * RADFAC;
 
-   if (rotate < 0) {  /* implies flipped object */
-      rotate = -rotate;
-      scale = -scale;
-   }
-   ascale = abs(scale);
+   yscale = abs(scale);		/* negative scale value implies flip in x only */
    
-   tmpa =  scale * (*cosine[rotate]);
-   tmpb = ascale * (*sine[rotate]);
-   tmpd = -scale * (*sine[rotate]);
-   tmpe = ascale * (*cosine[rotate]);
+   tmpa =  scale * cos(drot);
+   tmpb = yscale * sin(drot);
+   tmpd = -scale * sin(drot);
+   tmpe = yscale * cos(drot);
    
    ctm->c += ctm->a * position.x + ctm->b * position.y;
    ctm->f += ctm->d * position.x + ctm->e * position.y;
@@ -726,6 +674,25 @@ void UPreMultCTM(Matrix *ctm, XPoint position, float scale, short rotate)
    ctm->d = matd;
 }
 
+/*----------------------------------------------------------------------*/
+/* Direct Matrix-Matrix multiplication					*/
+/*----------------------------------------------------------------------*/
+
+void UPreMultCTMbyMat(Matrix *ctm, Matrix *pre)
+{
+   float mata, matd;
+
+   mata = pre->a * ctm->a + pre->d * ctm->b;
+   ctm->c += pre->c * ctm->a + pre->f * ctm->b;
+   ctm->b = pre->b * ctm->a + pre->e * ctm->b;
+   ctm->a = mata;
+
+   matd = pre->a * ctm->d + pre->d * ctm->e;
+   ctm->f += pre->c * ctm->d + pre->f * ctm->e;
+   ctm->e = pre->b * ctm->d + pre->e * ctm->e;
+   ctm->d = matd;
+}
+
 /*------------------------------------------------------------------------*/
 
 void UTransformbyCTM(Matrix *ctm, XPoint *ipoints, XPoint *points, short number)
@@ -1393,10 +1360,8 @@ void UDrawXDown(labelptr curlabel)
 {
    XPoint upt;
 
-   if (areastruct.pinpointon && curlabel->pin && curlabel->pin != INFO) {
-      UTransformbyCTM(DCTM, &curlabel->position, &upt, 1);
-      UDrawXAt(&upt);
-   }
+   UTransformbyCTM(DCTM, &curlabel->position, &upt, 1);
+   UDrawXAt(&upt);
 }
 
 /*-------------------------------------------------------------------------*/
@@ -1715,8 +1680,8 @@ void UDrawObject(objinstptr localdata, objectptr theobject, short level,
    newhierarchy->thisobject = theobject;
 
    UPushCTM();
-   UPreMultCTM(DCTM, localdata->position, localdata->scale,
-	 localdata->rotation);
+   if (localdata != NULL)
+      UPreMultCTM(DCTM, localdata->position, localdata->scale, localdata->rotation);
 
    /* do a quick test for intersection with the display window */
 
@@ -1769,17 +1734,22 @@ void UDrawObject(objinstptr localdata, objectptr theobject, short level,
 	     break;
    
           case(OBJECT):
-             UDrawObject(TOOBJINST(areagen), TOOBJINST(areagen)->thisobject,
+	     if ((!areastruct.editinplace) ||
+			(TOOBJINST(areagen)->thisobject != objectdata) ||
+			(TOOBJINST(areagen) != areastruct.topobject))
+                UDrawObject(TOOBJINST(areagen), TOOBJINST(areagen)->thisobject,
 			level + 1, curcolor);
 	     break;
    
   	  case(LABEL): 
 #ifdef SCHEMA
-	     if (level == 0 || TOLABEL(areagen)->pin == False)
+	     if (level == 0 || TOLABEL(areagen)->pin == False ||
+			(TOLABEL(areagen)->justify & PINVISIBLE))
 #endif
              UDrawString(localdata, TOLABEL(areagen), curcolor);
 #ifdef SCHEMA
-	     else if (level == 1 && TOLABEL(areagen)->pin == True)
+	     else if (level == 1 && TOLABEL(areagen)->pin &&
+			TOLABEL(areagen)->pin != INFO && areastruct.pinpointon)
 		UDrawXDown(TOLABEL(areagen));
 #endif
 	     break;
