@@ -260,7 +260,7 @@ void getsnapspace(Widget button, caddr_t clientdata, caddr_t calldata)
 
    getgeneric(savebutton, button, getsnapspace, (void *)floatptr);
    measurestr(*floatptr, buffer);
-   popupprompt(button, "Enter value:", buffer, setgrid, savebutton, False);
+   popupprompt(button, "Enter value:", buffer, setgrid, savebutton, NULL);
 }
 
 /*--------------------------------------------------------------*/
@@ -275,7 +275,7 @@ void getgridspace(Widget button, caddr_t clientdata, caddr_t calldata)
 
    getgeneric(savebutton, button, getgridspace, (void *)floatptr);
    measurestr(*floatptr, buffer);
-   popupprompt(button, "Enter value:", buffer, setgrid, savebutton, False);
+   popupprompt(button, "Enter value:", buffer, setgrid, savebutton, NULL);
 }
 
 /*----------------------------------------------------------------*/
@@ -595,7 +595,7 @@ void getkern(Widget button, caddr_t nulldata, caddr_t calldata)
    }
 
    getgeneric(savebutton, button, getkern, strptr);
-   popupprompt(button, "Enter Kern X,Y:", buffer, setkern, savebutton, False);
+   popupprompt(button, "Enter Kern X,Y:", buffer, setkern, savebutton, NULL);
 }
 
 /*--------------------------------------------------------------*/
@@ -640,7 +640,7 @@ void getdscale(Widget button, caddr_t nulldata, caddr_t calldata)
 
    getgeneric(savebutton, button, getdscale, ptptr);
    sprintf(buffer, "%d:%d", ptptr->x, ptptr->y);
-   popupprompt(button, "Enter Scale:", buffer, setdscale, savebutton, False);
+   popupprompt(button, "Enter Scale:", buffer, setdscale, savebutton, NULL);
 }
 
 /*----------------------------------------------------------------*/
@@ -663,9 +663,9 @@ void setdscale(Widget w, XPoint *dataptr)
    }
 }
 
-/*----------------------------------------------------------------*/
-/* Generate the popup dialog for getting text scale.		  */ 
-/*----------------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Generate the popup dialog for getting text scale.		*/ 
+/*--------------------------------------------------------------*/
 
 void gettsize(Widget button, caddr_t nulldata, caddr_t calldata)
 {
@@ -712,13 +712,13 @@ void gettsize(Widget button, caddr_t nulldata, caddr_t calldata)
    }
    if (settext != NULL) {
       getgeneric(savebutton, button, gettsize, settext);
-      popupprompt(button, "Enter text scale:", buffer, settsize, savebutton, False);
+      popupprompt(button, "Enter text scale:", buffer, settsize, savebutton, NULL);
    }
    else {
       sprintf(buffer, "%4.2f", *floatptr);
       getgeneric(savebutton, button, gettsize, floatptr);
       popupprompt(button,
-	    "Enter default text scale:", buffer, setfloat, savebutton, False);
+	    "Enter default text scale:", buffer, setfloat, savebutton, NULL);
    }
 }
 
@@ -774,7 +774,7 @@ void getosize(Widget button, caddr_t clientdata, caddr_t calldata)
    flval = setobj->scale;
    getgeneric(savebutton, button, getosize, setobj);
    sprintf(buffer, "%4.2f", flval);
-   popupprompt(button, "Enter object scale:", buffer, setosize, savebutton, False);
+   popupprompt(button, "Enter object scale:", buffer, setosize, savebutton, NULL);
 }
 
 /*----------------------------------------------------------------*/
@@ -791,7 +791,7 @@ void getwirewidth(Widget button, caddr_t clientdata, caddr_t calldata)
    getgeneric(savebutton, button, getwirewidth, widthptr);
    sprintf(buffer, "%4.2f", *widthptr / 2.0);
    popupprompt(button, "Enter new global linewidth:", buffer, setwidth,
-	savebutton, False);
+	savebutton, NULL);
 }
 
 /*----------------------------------------------------------------*/
@@ -875,12 +875,12 @@ void getwwidth(Widget button, caddr_t clientdata, caddr_t calldata)
    if (osel == areastruct.selectlist + areastruct.selects) {
       sprintf(buffer, "%4.2f", areastruct.linewidth);
       popupprompt(button, "Enter new default line width:", buffer, setwwidth,
-		savebutton, False);
+		savebutton, NULL);
    }
    else {
       sprintf(buffer, "%4.2f", flval);
       popupprompt(button, "Enter new line width:", buffer, setwwidth,
-		savebutton, False);
+		savebutton, NULL);
    }
 }
 
@@ -895,7 +895,7 @@ void getfloat(Widget button, float *floatptr, caddr_t calldata)
 
    getgeneric(savebutton, button, getfloat, floatptr);
    sprintf(buffer, "%4.2f", *floatptr);
-   popupprompt(button, "Enter value:", buffer, setfloat, savebutton, False);
+   popupprompt(button, "Enter value:", buffer, setfloat, savebutton, NULL);
 }
 
 /*----------------------------------------------------------------*/
@@ -934,7 +934,7 @@ void setpagelabel(Widget w, char *dataptr)
 {
    short i;
 
-   /* whitespace and non-printing characters not allowed */
+   /* Whitespace and non-printing characters not allowed */
 
    for (i = 0; i < strlen(_STR2); i++) {
       if ((!isprint(_STR2[i])) || (isspace(_STR2[i]))) {
@@ -942,11 +942,19 @@ void setpagelabel(Widget w, char *dataptr)
          Wprintf("Replaced illegal whitespace in name with underscore");
       }
    }
+
    if (!strcmp(dataptr, _STR2)) return; /* no change in string */
    if (strlen(_STR2) == 0)
       sprintf(objectdata->name, "Page %d", areastruct.page + 1);
    else
       sprintf(objectdata->name, "%.79s", _STR2);
+
+#ifdef SCHEMA
+   /* For schematics, all pages with associations to symbols must have	*/
+   /* unique names.							*/
+   if (objectdata->symschem != NULL) checkpagename(objectdata);
+#endif
+
    printname(objectdata);
    renamepage(areastruct.page);
 }
@@ -1107,6 +1115,7 @@ void setcolorscheme(Boolean boolvalue)
       OFFBUTTONCOLOR = appdata.buttonpix;
       SELECTCOLOR = appdata.selectpix;
       QUERYCOLOR = appdata.querypix;
+      FILTERCOLOR = appdata.filterpix;
       GRIDCOLOR = appdata.gridpix;
       SNAPCOLOR = appdata.snappix;
       AXESCOLOR = appdata.axespix;
@@ -1120,6 +1129,7 @@ void setcolorscheme(Boolean boolvalue)
       OFFBUTTONCOLOR = appdata.buttonpix2;
       SELECTCOLOR = appdata.selectpix2;
       QUERYCOLOR = appdata.querypix2;
+      FILTERCOLOR = appdata.filterpix2;
       GRIDCOLOR = appdata.gridpix2;
       SNAPCOLOR = appdata.snappix2;
       AXESCOLOR = appdata.axespix2;
@@ -1795,7 +1805,7 @@ void addnewcolor(Widget w, caddr_t clientdata, caddr_t calldata)
 
    getgeneric(savebutton, w, addnewcolor, NULL);
    popupprompt(w, "Enter color name or #rgb or r,g,b:", "\0", setnewcolor,
-	savebutton, False);
+	savebutton, NULL);
 }
 
 /*------------------------------------------------------*/
@@ -1842,12 +1852,23 @@ void setfontmarks(short fvalue, short jvalue)
 
       toggleexcl(w, Justifs, XtNumber(Justifs));
 
+      /* Flip Invariance property */
       w = JustificationFlipInvariantButton;
       if (jvalue & FLIPINV)
          XtSetArg(wargs[0], XtNsetMark, True);
       else
          XtSetArg(wargs[0], XtNsetMark, False);
       XtSetValues(w, wargs, 1);
+
+#ifdef SCHEMA
+      /* Pin visibility property */
+      w = NetlistPinVisibilityButton;
+      if (jvalue & PINVISIBLE)
+         XtSetArg(wargs[0], XtNsetMark, True);
+      else
+         XtSetArg(wargs[0], XtNsetMark, False);
+      XtSetValues(w, wargs, 1);
+#endif
    }
 }
 
@@ -1933,6 +1954,28 @@ void doflipflip(Widget w, labelptr settext)
    if (w != NULL) toggle(w, &boolval, NULL);
 }
 
+/*----------------------------------------------------------------*/
+/* Change pin visibility on the label passed as 2nd parameter	  */
+/*----------------------------------------------------------------*/
+
+void dopinvisible(Widget w, labelptr settext)
+{
+   short newjust;
+   Boolean boolval;
+   u_short value;
+
+   if (settext != NULL && settext->pin) {
+      boolval = (settext->justify & PINVISIBLE) ? True : False;
+      value = boolval ? 0 : PINVISIBLE;
+      newjust = (settext->justify & (~PINVISIBLE)) | value;
+
+      undrawtext(settext);
+      settext->justify = newjust;
+      redrawtext(settext);
+      if (w != NULL) toggle(w, &boolval, NULL);
+   }
+}
+
 /*----------------------------------------------------------------*/
 /* Set the justification for the label passed as 3rd parameter	  */
 /*----------------------------------------------------------------*/
@@ -1944,9 +1987,11 @@ void setjust(Widget w, u_int value, labelptr settext, short mode)
    if (settext != NULL) {
 
       if (mode == 1)
-         newjust = (settext->justify & (FLIPINV | TOP | NOTBOTTOM)) | value;
+         newjust = (settext->justify & (PINVISIBLE | FLIPINV | TOP | NOTBOTTOM))
+			| value;
       else
-         newjust = (settext->justify & (FLIPINV | RIGHT | NOTLEFT)) | value;
+         newjust = (settext->justify & (PINVISIBLE | FLIPINV | RIGHT | NOTLEFT))
+			| value;
 
       undrawtext(settext);
       settext->justify = newjust;
@@ -1954,9 +1999,11 @@ void setjust(Widget w, u_int value, labelptr settext, short mode)
    }
    else {
       if (mode == 1)
-         newjust = (areastruct.justify & (FLIPINV | TOP | NOTBOTTOM)) | value;
+         newjust = (areastruct.justify & (PINVISIBLE | FLIPINV | TOP | NOTBOTTOM))
+			| value;
       else
-         newjust = (areastruct.justify & (FLIPINV | RIGHT | NOTLEFT)) | value;
+         newjust = (areastruct.justify & (PINVISIBLE | FLIPINV | RIGHT | NOTLEFT))
+			| value;
       areastruct.justify = newjust;
    }
    if (w != NULL) toggleexcl(w, Justifs, XtNumber(Justifs));
@@ -2048,6 +2095,35 @@ void setflipinv(Widget w, u_int value, caddr_t nulldata)
    }
 }
 
+/*----------------------------------------------------------------*/
+/* Set pin visibility on all selected pins			  */ 
+/*----------------------------------------------------------------*/
+
+void setpinvisible(Widget w, u_int value, caddr_t nulldata)
+{
+   short *fselect;
+   labelptr settext;
+   short labelcount = 0;
+
+   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+      settext = *((labelptr *)EDITPART);
+      if (settext->pin)
+	 dopinvisible(w, settext);
+   }
+   else {
+      for (fselect = areastruct.selectlist; fselect < areastruct.selectlist +
+            areastruct.selects; fselect++) {
+         if (SELECTTYPE(fselect) == LABEL) {
+            labelcount++;
+            settext = SELTOLABEL(fselect);
+	    if (settext->pin)
+               dopinvisible(NULL, settext);
+         }
+      }
+      objectdeselect();
+   }
+}
+
 /*---------------------------*/
 /* Set polygon editing style */
 /*---------------------------*/
@@ -2090,7 +2166,7 @@ void addnewfont(Widget w, caddr_t clientdata, caddr_t calldata)
    char *tempstr = malloc(2 * sizeof(char));
    tempstr[0] = '\0';
    getgeneric(savebutton, w, addnewfont, tempstr);
-   popupprompt(w, "Enter font name:", tempstr, locloadfont, savebutton, False);
+   popupprompt(w, "Enter font name:", tempstr, locloadfont, savebutton, NULL);
 }
 
 /*----------------------------------------------------------------*/
