@@ -1,6 +1,6 @@
 /*----------------------------------------------------------------------*/
 /* menudep.c								*/
-/*   Copyright (c) 2000  Tim Edwards, Johns Hopkins University        	*/
+/*   Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	*/
 /*----------------------------------------------------------------------*/
 /* Parse file menus.h and create file of definitions for use by		*/
 /* "NameToWidget" to find the menu in the widget menu tree.		*/
@@ -23,13 +23,13 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Xutil.h>
 
 #include "xcircuit.h"
-#include "menudefs.h"
 
 #define _MENUDEP
 #include "menus.h"
@@ -38,6 +38,7 @@ int menucount;
 FILE *mp;
 
 /*----------------------------------------------------------------------*/
+
 char *normalize(char *textin)
 {
    char *tptr = textin;
@@ -68,6 +69,7 @@ char *normalize(char *textin)
 }
 
 /*----------------------------------------------------------------------*/
+
 void log_entry(char *menuname, char *buttonname)
 {
    char *n1, *n2;
@@ -83,6 +85,7 @@ void log_entry(char *menuname, char *buttonname)
 }
 
 /*----------------------------------------------------------------------*/
+
 void toolbar_entry(char *buttonname)
 {
    char *n1;
@@ -92,6 +95,7 @@ void toolbar_entry(char *buttonname)
 }
 
 /*----------------------------------------------------------------------*/
+
 void searchsubmenu(char *menuname, menuptr buttonmenu, int arraysize)
 {
    menuptr p;
@@ -105,7 +109,8 @@ void searchsubmenu(char *menuname, menuptr buttonmenu, int arraysize)
 }
 
 /*----------------------------------------------------------------------*/
-main()
+
+int main()
 {
    short i;
 
