@@ -194,20 +194,22 @@ menustruct FilterMenu[] = {
 };
 
 menustruct RotateMenu[] = {
-	{"Flip Horizontal (f)", action(startrotate, Number(64))},
-	{"Flip Vertical (F)", action(startrotate, Number(65))},
+	{"Flip Horizontal (f)", action(startrotate, Number(512))},
+	{"Flip Vertical (F)", action(startrotate, Number(1024))},
 	{" ", noaction},
-	{"Rotate CW 90", action(startrotate, Number(18))},
-	{"Rotate CW 45", action(startrotate, Number(9))},
-	{"Rotate CW 30", action(startrotate, Number(6))},
-	{"Rotate CW 15 (r)", action(startrotate, Number(3))},
-	{"Rotate CW 5 (o)", action(startrotate, Number(1))},
+	{"Rotate CW 90", action(startrotate, Number(90))},
+	{"Rotate CW 45", action(startrotate, Number(45))},
+	{"Rotate CW 30", action(startrotate, Number(30))},
+	{"Rotate CW 15 (r)", action(startrotate, Number(15))},
+	{"Rotate CW 5 (o)", action(startrotate, Number(3))},
+	{"Rotate CW 1", action(startrotate, Number(1))},
 	{" ", noaction},
-	{"Rotate CCW 90", action(startrotate, Number(-18))},
-	{"Rotate CCW 45", action(startrotate, Number(-9))},
-	{"Rotate CCW 30", action(startrotate, Number(-6))}, 
-	{"Rotate CCW 15 (R)", action(startrotate, Number(-3))},
-	{"Rotate CCW 5 (O)", action(startrotate, Number(-1))},
+	{"Rotate CCW 90", action(startrotate, Number(-90))},
+	{"Rotate CCW 45", action(startrotate, Number(-45))},
+	{"Rotate CCW 30", action(startrotate, Number(-30))}, 
+	{"Rotate CCW 15 (R)", action(startrotate, Number(-15))},
+	{"Rotate CCW 5 (O)", action(startrotate, Number(-5))},
+	{"Rotate CCW 1", action(startrotate, Number(-1))},
 };
 
 menustruct GridMenu[] = {
@@ -235,6 +237,7 @@ menustruct SchemaMenu[] = {
 	{"Make Info Pin (I)", action(starttext, Number(INFO))},
 	{"Make Global Pin (G)", action(starttext, Number(GLOBAL))},
 	{"Convert Label to...", submenu(PinConvert)},
+	{"Pin Visibility", action(setpinvisible, Number(PINVISIBLE))},
 /*	{"Make Object Non-Schematic", action(maketrivial, NULL)}, */
 	{"Make Matching Symbol", action(dobeforeswap, NULL)},
 	{"Associate with Symbol", action(startschemassoc, NULL)},
@@ -249,11 +252,12 @@ menustruct SchemaMenu[] = {
 #endif
 
 menustruct WindowMenu[] = {
-	{"Zoom In (Z)", action(zoominrefresh, Number(0))},
+	{"Zoom In (Z)", action(zoominrefresh, NULL)},
 	{"Zoom Out (z)", action(zoomoutrefresh, NULL)},
 	{"Zoom Box", action(zoombox, NULL)},
 	{"Full View (v)", action(zoomview, NULL)},
 	{"Center Pan (p)",action(centerpan, NULL)},
+	{"Refresh ( )", action(refresh, NULL)},
 	{" ", noaction},
 	{"Library Directory (L)", action(startcatalog, Number(LIBLIB))},
 	{"Goto Library", submenu(Libraries)},
@@ -281,6 +285,7 @@ menustruct FileMenu[] = {
 menustruct OptionMenu[] = {
 	{"Alt Colors", action(inversecolor, &areastruct.invert)},
 	{"Show Bounding box", action(toggle, &areastruct.bboxon)},
+	{"Edit in place", setaction(toggle, &areastruct.editinplace)},
 #ifdef SCHEMA
 	{"Show Pin Positions", action(toggle, &areastruct.pinpointon)},
 	{"Enable XSchema", action(doxschema, NULL)},
@@ -304,7 +309,6 @@ menustruct EditMenu[] = {
 	{"Rotate/Flip", submenu(RotateMenu)},
 	{"Deselect (x)", action(startdesel, NULL)},
 	{"Select filter", submenu(FilterMenu)},
-	{"Refresh ( )", action(refresh, NULL)},
 /*	{"Array", noaction}, */
 	{"Push selected (>)", action(startpush, NULL)},
 	{"Pop hierarchy (<)", action(popobject, Number(0))},
@@ -349,9 +353,9 @@ toolbarstruct ToolBar[] = {
 		"rotate 15 degrees clockwise"},
 	{"RotN",  ccw_xpm, toolaction(startrotate, Number(-3)),
 		"rotate 15 degrees counterclockwise"},
-	{"HFlip",  hf_xpm, toolaction(startrotate, Number(64)),
+	{"HFlip",  hf_xpm, toolaction(startrotate, Number(512)),
 		"flip horizontal"},
-	{"VFlip",  vf_xpm, toolaction(startrotate, Number(65)),
+	{"VFlip",  vf_xpm, toolaction(startrotate, Number(1024)),
 		"flip vertical"},
 	{"Push",  pu2_xpm, toolaction(startpush, NULL),
 		"push (edit object)"},
@@ -375,7 +379,7 @@ toolbarstruct ToolBar[] = {
 		"go to library directory"},
 	{"Pagedir",pl_xpm, toolaction(startcatalog, Number(PAGELIB)),
 		"go to page directory"},
-	{"ZoomI",  z4_xpm, toolaction(zoomin, Number(0)),
+	{"ZoomI",  z4_xpm, toolaction(zoomin, NULL),
 		"zoom in"},
 	{"ZoomO",  z5_xpm, toolaction(zoomout, NULL),
 		"zoom out"},
