@@ -18,6 +18,10 @@
 #include <malloc.h>
 #endif
 
+#ifdef HAVE_PYTHON
+#include <Python.h>
+#endif
+
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 
@@ -33,6 +37,10 @@
 /*----------------------------------------------------------------------*/
 #include "prototypes.h"
 
+#ifdef HAVE_PYTHON
+extern PyObject *PyGetStringParts(stringpart *);
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Externally declared global variables					*/
 /*----------------------------------------------------------------------*/
@@ -126,6 +134,15 @@ void destroynets()
 
 void gennet(char *mode, char *suffix)
 {
+   /* Run cleartraversed() on all types, even though only hierarchical	*/
+   /* netlists require it.  Here we can check for infinite recursion	*/
+   /* and stop it before it becomes an uncorrectable problem.		*/
+
+   if (cleartraversed(objectdata, 0) == -1) {
+      Wprintf("Error:  Check for recursion in circuit!");
+      return;
+   }
+
    createnets(False, mode);
    
    if (objectdata->netlist != NULL)
@@ -790,7 +807,7 @@ int translateup(int rnet, objectptr thisobj, objectptr nextobj, objinstptr nexti
 {
    PortlistPtr nport;
    CalllistPtr ccall;
-   int portid;
+   int portid = 0;
    int upnet = 0;
 
    for (nport = nextobj->portlist; nport != NULL; nport = nport->next) {
@@ -2314,6 +2331,196 @@ void writehierarchy(objectptr cschem, CalllistPtr cfrom, FILE *fp)
       fprintf(fp, ".ends\n\n");
 }
 
+/*----------------------------------------------------------------------*/
+/* Create generic netlist in the Python interpreter variable space	*/
+/*----------------------------------------------------------------------*/
+
+#ifdef HAVE_PYTHON
+
+static PyObject *pyparseinfo(objectptr cschem)
+{
+   genericptr *pgen;
+   labelptr plabel;
+
+   PyObject *rlist = PyList_New(0);
+
+   for (pgen = cschem->plist; pgen < cschem->plist + cschem->parts; pgen++) {
+      if ((*pgen)->type == LABEL) {
+	 plabel = TOLABEL(pgen);
+         if (plabel->pin == INFO)
+	    PyList_Append(rlist, PyGetStringParts(plabel->string));
+      }
+   }
+   return rlist;
+}
+
+/*----------------------------------------------------------------------*/
+/* Write global variables to Python list "globalnets"			*/
+/*----------------------------------------------------------------------*/
+
+PyObject *pyglobals(objectptr cschem)
+{
+   NetlistPtr gptr;
+   labelptr gpin;
+   char *snew;
+   PyObject *glist;
+   int llen = 0;
+
+   for (gptr = globallist; gptr != NULL; gptr = gptr->next, llen++);
+   glist = PyList_New(llen);
+
+   llen = 0;
+   for (gptr = globallist; gptr != NULL; gptr = gptr->next, llen++) {
+      gpin = gptr->labels->label;	/* Only print first label in list */
+      PyList_SetItem(glist, llen, PyGetStringParts(gpin->string));
+   }
+   return glist;
+}
+
+/*----------------------------------------------------------------------*/
+/* Write a generic hierarchical netlist into Python list "subcircuits"	*/
+/*----------------------------------------------------------------------*/
+
+void pyhierarchy(objectptr cschem, CalllistPtr cfrom, PyObject *cktlist)
+{
+   CalllistPtr calllist = cschem->calllist;
+   PortlistPtr portlist, plist;
+   int netid, pnet, portid, length, plen, i;
+   PyObject *pyports, *pycalls, *pynewcall;
+   PyObject *pydevs, *pyparams, *subckt, *newdev;
+   oparamptr paramlist;
+
+   /* make sure that all the subcircuits have been written first */
+
+   for (; calllist != NULL; calllist = calllist->next) {
+      if (calllist->callobj->traversed == False) {
+         calllist->callobj->traversed = True;
+	 pyhierarchy(calllist->callobj, calllist, cktlist);
+      }
+   }
+
+   /* Write own subcircuit netlist */
+   subckt = PyDict_New();
+
+   /* Write the name */
+   PyDict_SetItem(subckt, PyString_FromString("name"),
+		PyString_FromString(cschem->name));
+
+   /* Write the list of ports */
+   if ((portlist = cschem->portlist) != NULL) {
+
+      /* List of ports in subcircuit. 			  	  */
+      /* Each parameter connects to a net which may have multiple */
+      /* names, so find the net associated with the parameter     */
+      /* and convert it back into a pin name in the usual manner  */
+      /* using nettopin().					  */
+
+      pyports = PyList_New(0);
+      for (; portlist != NULL; portlist = portlist->next) {
+         netid = portlist->netid;
+	 PyList_Append(pyports, PyGetStringParts(nettopin(netid, cschem, NULL)));
+      }
+      PyDict_SetItem(subckt, PyString_FromString("ports"), pyports);
+   }
+
+   /* Write the list of parameters */
+
+   if (cschem->params != NULL) {
+      pyparams = PyList_New(0);
+
+      for (i = 0; i < cschem->num_params; i++) {
+	 paramlist = *(cschem->params + i);
+	 if (paramlist->type == XC_SHORT)
+	    PyList_Append(pyparams,
+			PyInt_FromLong((long)paramlist->parameter.svalue));
+	 else if (paramlist->type == XC_STRING)
+	    PyList_Append(pyparams,
+			PyGetStringParts(paramlist->parameter.string));
+      }
+      PyDict_SetItem(subckt, PyString_FromString("parameters"), pyparams);
+   }
+
+   /* Write the list of calls to subcircuits */
+
+   if ((calllist = cschem->calllist) != NULL) {
+      pycalls = PyList_New(0);
+      for (; calllist != NULL; calllist = calllist->next) {
+
+         pynewcall = PyDict_New();
+
+         PyDict_SetItem(pynewcall, PyString_FromString("name"),
+		PyString_FromString(calllist->callobj->name));
+
+         /* Log any local parameter instances */
+         if (calllist->callinst->params != NULL) {
+	    pyparams = PyList_New(0);
+
+	    for (i = 0; i < calllist->callobj->num_params; i++) {
+	       paramlist = *(calllist->callinst->params + i);
+	       if (paramlist->type == XC_SHORT)
+	          PyList_Append(pyparams,
+			PyInt_FromLong((long)paramlist->parameter.svalue));
+	       else if (paramlist->type == XC_STRING)
+	          PyList_Append(pyparams,
+			PyGetStringParts(paramlist->parameter.string));
+	    }
+	    PyDict_SetItem(pynewcall, PyString_FromString("parameters"),
+			pyparams);
+         }
+
+         /* The object's definition lists calls in the order of the object's	*/
+         /* port list.  Therefore, use this order to find the port list.  	*/
+	 /* Unconnected ports will be NULL entries in the list (?).		*/
+
+         if (calllist->callobj->portlist != NULL) {
+            pyports = PyList_New(0);
+	    for (portlist = calllist->callobj->portlist; portlist != NULL;
+			portlist = portlist->next) {
+	       portid = portlist->portid;
+	       for (plist = calllist->ports; plist != NULL; plist = plist->next)
+	          if (plist->portid == portlist->portid)
+		     break;
+
+	       pnet = (plist == NULL) ?  netmax(cschem) + 1 : plist->netid;
+
+	       PyList_Append(pyports, 
+			PyGetStringParts(nettopin(pnet, cschem, NULL)));
+            }
+	    PyDict_SetItem(pynewcall, PyString_FromString("ports"), pyports);
+         }
+         PyList_Append(pycalls, pynewcall);
+      }
+      PyDict_SetItem(subckt, PyString_FromString("calls"), pycalls);
+   }
+
+   /* If the object has info labels, write a device list.	*/
+   /* Check both the schematic and its symbol for info labels.	*/
+
+   pydevs = PyList_New(0);
+   if (cschem->symschem != NULL) {
+      newdev = pyparseinfo(cschem->symschem);
+      PyList_Append(pydevs, newdev);
+   }
+   newdev = pyparseinfo(cschem);
+   PyList_Append(pydevs, newdev);
+   PyDict_SetItem(subckt, PyString_FromString("devices"), pydevs);
+
+   PyList_Append(cktlist, subckt);
+}
+
+/*----------------------------------------------------------------------*/
+
+PyObject *pytoplevel(objectptr cschem)
+{
+   PyObject *cktlist;
+
+   cktlist = PyList_New(0);
+   pyhierarchy(cschem, NULL, cktlist);
+   return cktlist;
+}
+
+#endif	/* HAVE_PYTHON */
+
 /*----------------------------------------------------------------------*/
 /* Write a netlist depending on the mode chosen				*/
 /*----------------------------------------------------------------------*/
@@ -2349,7 +2556,6 @@ void writenet(objectptr cschem, char *mode, char *suffix)
    if (!strcmp(mode, "spice")) {
       fprintf(fp, "*SPICE circuit \"%s\" from XCircuit v%3.2f\n\n",
 		cschem->name, version);
-      cleartraversed(cschem);
       writeglobals(cschem, fp);
       writehierarchy(cschem, NULL, fp);
    }
@@ -2710,7 +2916,7 @@ void freenetlist(objectptr cschem)
 /* Clear the "traversed" flag in all objects of the hierarchy.		*/
 /*----------------------------------------------------------------------*/
 
-void cleartraversed(objectptr cschem)
+int cleartraversed(objectptr cschem, int level)
 {
    genericptr *cgen;
    objinstptr cinst;
@@ -2720,6 +2926,8 @@ void cleartraversed(objectptr cschem)
    /* Use the parts list of the object, not the calllist, because calls */
    /* may have been removed.						*/
 
+   if (level == HIERARCHY_LIMIT) return -1;
+
    for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
       if ((*cgen)->type == OBJECT) {
  	 cinst = TOOBJINST(cgen);
@@ -2729,12 +2937,17 @@ void cleartraversed(objectptr cschem)
 	 else
 	    callobj = cinst->thisobject;
 	
-	 /* Don't infinitely recurse if object is on its own schematic */
+	 /* Don't infinitely recurse if object is on its own schematic	  */
+	 /* However, we need to take a stab at more subtle recursion, too */
 
-	 if (callobj != cschem) cleartraversed(callobj);
+	 if (callobj != cschem)
+	    if (cleartraversed(callobj, level + 1) == -1)
+	       return -1;
       }
    }
    cschem->traversed = False;
+
+   return 0;
 }
 
 /*-------------------------------------------------------------------------*/
