@@ -499,8 +499,7 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
    genericptr *pgen;
    labelptr plab;
    oparamptr ops;
-   stringpart *strptr, *lastpart, *nextpart, *endpart;
-   stringpart *subs = NULL, *newstr;
+   stringpart *strptr, *lastpart, *nextpart, *endpart, *newstr, *subs;
    int p, j, k;
    Boolean is_last = True;
    
@@ -541,6 +540,7 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
 
    /* Copy the default parameter into the place we are unparameterizing */
 
+   newstr = NULL;
    newstr = stringcopy(subs);
 
    /* Delete the "PARAM_END" off of the copied string and link it into	*/
@@ -565,9 +565,8 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
    free(strptr);
 
    /* Merge strings at boundaries, if possible. */
-   mergestring(lastpart);
-   strptr = lastpart;
    mergestring(endpart);
+   mergestring(lastpart);
 
    redrawtext(thislabel);
    if (!is_last) return;
