@@ -134,12 +134,12 @@ extern void endfiletrack(Widget, caddr_t, XCrossingEvent *);
 extern char *getcrashfilename();
 extern void crashrecover();
 extern void findcrashfiles();
-extern void listfiles(Widget, caddr_t, caddr_t);
-extern void newfilelist(Widget, Widget);
-extern void fileselect(Widget, Widget, XButtonEvent *);
+extern void listfiles(Widget, popupstruct *, caddr_t);
+extern void newfilelist(Widget, popupstruct *);
+extern void fileselect(Widget, popupstruct *, XButtonEvent *);
 extern void showlscroll(Widget, caddr_t, caddr_t);
-extern void draglscroll(Widget, Widget, XButtonEvent *);
-extern void genfilelist(Widget, Widget, Dimension);
+extern void draglscroll(Widget, popupstruct *, XButtonEvent *);
+extern void genfilelist(Widget, popupstruct *, Dimension);
 extern int lookdirectory(char *);
 
 /* from files.c: */
@@ -263,6 +263,9 @@ extern void zoomview(Widget, caddr_t, caddr_t);
 extern void UDrawSimpleLine(objinstptr, XPoint *, XPoint *);
 extern void UDrawLine(objinstptr, XPoint *, XPoint *);
 extern void UDrawX(labelptr);
+#ifdef SCHEMA
+extern void UDrawXDown(labelptr);
+#endif
 extern void pinadjust(short, short *, short *, short);
 extern void UDrawTextLine(labelptr, short);
 extern void UDrawTLine(labelptr);
@@ -394,11 +397,13 @@ extern void setfontmarks(short, short);
 extern void startparam(Widget, u_int, caddr_t);
 extern void startunparam(Widget, u_int, caddr_t);
 extern void setdefaultfontmarks();
+extern void dopinvisible(Widget, labelptr);
 extern void doflipflip(Widget, labelptr);
 extern void setjust(Widget, u_int, labelptr, short);
 extern void setvjust(Widget, u_int, caddr_t);
 extern void sethjust(Widget, u_int, caddr_t);
 extern void setflipinv(Widget, u_int, caddr_t);
+extern void setpinvisible(Widget, u_int, caddr_t);
 extern void boxedit(Widget, u_int, caddr_t);
 extern void locloadfont(Widget, char *);
 extern void addnewfont(Widget, caddr_t, caddr_t);
@@ -503,8 +508,7 @@ extern void exit_interpreter();
 #endif
 
 #ifdef HAVE_XPM
-extern int pytoolbuttons();
-extern Widget pytoolbutton(int);
+extern Widget *pytoolbuttons(int *);
 #endif
 
 /* from rcfile.c: */
@@ -620,7 +624,7 @@ extern void fontcatbutton(XButtonEvent *);
 extern void printeventmode();
 extern void Wprintf(char *);
 extern void popupprompt(Widget, char *, char *, void (*function)(),
-                        buttonsave *, Boolean);
+                        buttonsave *, const char *);
 extern void getproptext(Widget, propstruct *, caddr_t);
 extern int rgb_alloccolor(int, int, int);
 extern void addtocolorlist(Widget, int);
