@@ -115,6 +115,7 @@ extern Widget menuwidgets[];
 extern Display *dpy;
 extern Window win;
 extern keybinding *keylist;
+extern int number_colors;
 extern colorindex *colorlist;
 extern ApplicationData appdata;
 #ifdef HAVE_XPM
@@ -123,7 +124,7 @@ extern Widget toolbar;
 
 /*----------------------------------------------------------------------*/
 
-short flags;
+short flags = -1;
 
 #define LIBOVERRIDE     1
 #define LIBLOADED       2
@@ -160,8 +161,8 @@ void pypromptvalue(Widget w, char *functionptr)
 /* python function.						*/
 /*--------------------------------------------------------------*/
 
-static PyObject *pypopupprompt(PyObject *self, PyObject *args, Boolean dofile,
-	char *displaystr)
+static PyObject *pypopupprompt(PyObject *self, PyObject *args,
+	const char *dofile, char *displaystr)
 {
    const char *pfunction = NULL;
    const char *prompt = NULL;
@@ -184,12 +185,12 @@ static PyObject *pypopupprompt(PyObject *self, PyObject *args, Boolean dofile,
 
 static PyObject *xc_popupprompt(PyObject *self, PyObject *args)
 {
-   return pypopupprompt(self, args, False, "");
+   return pypopupprompt(self, args, NULL, "");
 }
 
 static PyObject *xc_filepopup(PyObject *self, PyObject *args)
 {
-   return pypopupprompt(self, args, True, "");
+   return pypopupprompt(self, args, "", "");
 }
 
 static PyObject *xc_simplepopup(PyObject *self, PyObject *args)
@@ -319,6 +320,78 @@ int string_to_type(const char *etype)
       return -1;
 }
 
+/*--------------------------------------------------------------*/
+/* Convert color index to 3-tuple and vice versa		*/
+/* We assume that this color exists in the color table.		*/
+/*--------------------------------------------------------------*/
+
+PyObject *PyIndexToRGB(int cidx)
+{
+   int i;
+   PyObject *RGBTuple;
+
+   if (cidx < 0) {	/* Handle "default color" */
+      return PyString_FromString("Default");
+   }
+
+   for (i = 0; i < number_colors; i++) {
+      if (cidx == colorlist[i].color.pixel) {
+	 RGBTuple = PyTuple_New(3);
+	 PyTuple_SetItem(RGBTuple, 0,
+		PyInt_FromLong((long)(colorlist[i].color.red / 256)));
+	 PyTuple_SetItem(RGBTuple, 1,
+		PyInt_FromLong((long)(colorlist[i].color.green / 256)));
+	 PyTuple_SetItem(RGBTuple, 2,
+		PyInt_FromLong((long)(colorlist[i].color.blue / 256)));
+	 return RGBTuple;
+      }
+   }
+   PyErr_SetString(PyExc_TypeError, "invalid or unknown color index");
+   return NULL;
+}
+
+/*--------------------------------------------------------------*/
+/* Convert color 3-tuple (RGB) or name to a color index		*/
+/*--------------------------------------------------------------*/
+
+int PyRGBToIndex(PyObject *cobj)
+{
+   PyObject *tobj;
+   int ccolor, r, g, b;
+
+   if (PyTuple_Check(cobj) && PyTuple_Size(cobj) == 3) {  /* RGB 3-tuple */
+      tobj = PyTuple_GetItem(cobj, 0);
+      if (PyFloat_Check(tobj)) {
+         r = (int)(65535 * (float)PyFloat_AsDouble(tobj));
+         g = (int)(65535 * (float)PyFloat_AsDouble(PyTuple_GetItem(cobj, 1)));
+         b = (int)(65535 * (float)PyFloat_AsDouble(PyTuple_GetItem(cobj, 2)));
+      }
+      else if (PyInt_Check(tobj)) {
+         r = (int)(256 * PyInt_AsLong(tobj));
+         g = (int)(256 * PyInt_AsLong(PyTuple_GetItem(cobj, 1)));
+         b = (int)(256 * PyInt_AsLong(PyTuple_GetItem(cobj, 2)));
+      }
+      else {
+         PyErr_SetString(PyExc_TypeError, "tuple components must be integer or float");
+         return -1;
+      }
+      ccolor = rgb_alloccolor(r, g, b);
+   }
+   else if (PyString_Check(cobj)) {  /* color name */
+      ccolor = xc_alloccolor(PyString_AsString(cobj));
+   }
+   else if (PyInt_Check(cobj)) {  /* index (for backward compatibility) */
+      ccolor = (int)PyInt_AsLong(cobj);
+   }
+   else {
+      PyErr_SetString(PyExc_TypeError, "argument must be a string or 3-tuple");
+      return -1;
+   }
+
+   addnewcolorentry(ccolor);
+   return ccolor;
+}
+
 /*--------------------------------------------------------------*/
 /* Convert a Python list to a stringpart* 			*/
 /*--------------------------------------------------------------*/
@@ -374,7 +447,7 @@ stringpart *PySetStringParts(PyObject *dval)
 	 }
 	 else if ((ditem = PyDict_GetItemString(litem, "Color")) != NULL) {
 	    newpart->type = FONT_COLOR;
-	    newpart->data.color = (int)PyInt_AsLong(ditem);
+	    newpart->data.color = PyRGBToIndex(ditem);
 	 }
 	 else if ((ditem = PyDict_GetItemString(litem, "Font Scale")) != NULL) {
 	    newpart->type = FONT_SCALE;
@@ -486,16 +559,12 @@ PyObject *PyGetStringParts(stringpart *thisstring)
 	    PyList_SetItem(lstr, i, sdict);
 	    break;
 	 case FONT_COLOR:
-	    sdict = PyDict_New();
-	    stup = PyTuple_New(3);
-	    PyTuple_SetItem(stup, 0, PyInt_FromLong(
-		(long)colorlist[strptr->data.color].color.red));
-	    PyTuple_SetItem(stup, 1, PyInt_FromLong(
-		(long)colorlist[strptr->data.color].color.green));
-	    PyTuple_SetItem(stup, 2, PyInt_FromLong(
-		(long)colorlist[strptr->data.color].color.blue));
-	    PyDict_SetItem(sdict, PyString_FromString("Color"), stup); 
-	    PyList_SetItem(lstr, i, sdict);
+	    stup = PyIndexToRGB(strptr->data.color);
+	    if (stup != NULL) {
+	       sdict = PyDict_New();
+	       PyDict_SetItem(sdict, PyString_FromString("Color"), stup); 
+	       PyList_SetItem(lstr, i, sdict);
+	    }
 	    break;
 	 case TABSTOP:
 	    PyList_SetItem(lstr, i, PyString_FromString("Tab Stop"));
@@ -879,11 +948,11 @@ static PyObject *xc_zoom(PyObject *self, PyObject *args)
 
    if (factor < 1.0) {
       areastruct.zoomfactor = 1.0 / factor;
-      zoomout(NULL, Number(0), NULL);
+      zoomout(NULL, NULL, NULL);
    }
    else {
       areastruct.zoomfactor = factor;
-      zoomin(NULL, Number(0), NULL);
+      zoomin(NULL, NULL, NULL);
    }
 
    areastruct.zoomfactor = save;
@@ -999,8 +1068,9 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
    PyDict_SetItem(rdict, PyString_FromString("type"),
         PyString_FromString(tstr));
    free(tstr);
-   PyDict_SetItem(rdict, PyString_FromString("color"), 
-	PyInt_FromLong((long)((*gelem)->color)));
+   lstr = PyIndexToRGB((*gelem)->color);
+   if (lstr != NULL)
+      PyDict_SetItem(rdict, PyString_FromString("color"), lstr);
 
    switch((*gelem)->type) {
       case LABEL:
@@ -1123,7 +1193,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
    /* to a number of different elements (position, color, etc.)	      */
 
    if ((dval = PyDict_GetItemString(attrdict, "color")) != NULL) {
-      (*gelem)->color = (short)PyInt_AsLong(dval);
+      (*gelem)->color = (short)PyRGBToIndex(dval);
    }
 
    if ((dval = PyDict_GetItemString(attrdict, "position")) != NULL) {
@@ -1515,13 +1585,14 @@ static PyObject *xc_font(PyObject *self, PyObject *args)
 
 static PyObject *xc_color(PyObject *self, PyObject *args)
 {
-   const char *colorname;
+   PyObject *pcolor;
+   int cidx;
 
-   if (!PyArg_ParseTuple(args, "s:color", &colorname))
+   if (!PyArg_ParseTuple(args, "O:color", &pcolor))
       return NULL;
 
-   addnewcolorentry(xc_alloccolor((char *)colorname));
-   return PyString_FromString(colorname);
+   cidx = PyRGBToIndex(pcolor);
+   return PyIndexToRGB(cidx);
 }
 
 /*--------------------------------------------------------------*/
@@ -1662,20 +1733,6 @@ int pybutton(Widget w, caddr_t clientdata, caddr_t calldata)
    return status;
 }
 
-/*--------------------------------------------------------------*/
-/* Callback procedure for Python-generated toolbar tool		*/
-/*--------------------------------------------------------------*/
-
-int pytool(Widget w, caddr_t clientdata, caddr_t calldata)
-{  
-   int status;
-
-   sprintf(_STR2, "tooldict[%d]()\n", w);
-   status = PyRun_SimpleString(_STR2);
-   refresh(NULL, NULL, NULL);
-   return status;
-}
-
 /*--------------------------------------------------------------*/
 /* Add a new button to the specified (by name) cascade menu	*/
 /*--------------------------------------------------------------*/
@@ -1717,6 +1774,20 @@ static PyObject *xc_newbutton(PyObject *self, PyObject *args)
 
 #ifdef HAVE_XPM
 
+/*--------------------------------------------------------------*/
+/* Callback procedure for Python-generated toolbar tool		*/
+/*--------------------------------------------------------------*/
+
+int pytool(Widget w, caddr_t clientdata, caddr_t calldata)
+{  
+   int status;
+
+   sprintf(_STR2, "tooldict[%d]()\n", w);
+   status = PyRun_SimpleString(_STR2);
+   refresh(NULL, NULL, NULL);
+   return status;
+}
+
 /*--------------------------------------------------------------*/
 /* Add a new tool to the toolbar				*/
 /*--------------------------------------------------------------*/
@@ -1768,21 +1839,12 @@ static PyObject *xc_newtool(PyObject *self, PyObject *args)
 }
 
 /*--------------------------------------------------------------*/
-/* Return the size of tooldict					*/
+/* Return a list of the widgets in tooldict			*/
 /*--------------------------------------------------------------*/
  
-int pytoolbuttons()
+Widget *pytoolbuttons(int *num_tools)
 {
-   return 0;
-}
-
-/*--------------------------------------------------------------*/
-/* Return the widget address of the Nth entry in tooldict	*/
-/*--------------------------------------------------------------*/
-
-Widget pytoolbutton(int index)
-{
-   return NULL;
+   return NULL;		/* No action, yet */
 }
 
 #endif
@@ -1912,6 +1974,7 @@ void execscript()
 
    flags = 0;
 
+   tilde_expand(_STR2);
    if ((fd = fopen(_STR2, "r")) != NULL) {
       PyRun_SimpleFile(fd, _STR2);
       refresh(NULL, NULL, NULL);
@@ -1940,6 +2003,7 @@ void loadrcfile()
       
    /* try first in current directory, then look in user's home directory */
       
+   tilde_expand(_STR2);
    if ((fd = fopen(_STR2, "r")) == NULL) {
       sprintf(_STR2, "%s/%s", userdir, USER_RC_FILE);
       fd = fopen(_STR2, "r");
