@@ -111,6 +111,65 @@ void doxschema(Widget w, caddr_t clientdata, caddr_t calldata)
    drawarea(NULL, NULL, NULL);
 }
 
+/*------------------------------------------------------------------------*/
+/* Ensure that a page name is unique (required for schematic association) */
+/*------------------------------------------------------------------------*/
+
+int checkpagename(objectptr thispageobj)
+{
+   int p, thispage;
+   char *objname = thispageobj->name;
+   Boolean changed;
+   Boolean update = False;
+   char *clnptr = NULL;
+   int n;
+
+   /* Check for ":n" suffix and prepare for possible update */
+   clnptr = strrchr(thispageobj->name, ':');
+   if (clnptr != NULL)
+      if (sscanf(clnptr + 1, "%d", &n) != 1)
+	 clnptr = NULL;
+
+   /* Find the page number of this page object */
+   for (p = 0; p < xobjs.pages; p++) {
+      if (xobjs.pagelist[p]->pageobj != NULL) {
+	 if (xobjs.pagelist[p]->pageobj == thispageobj) {
+	    thispage = p;
+	    break;
+	 }
+      }
+   }
+   if (p == xobjs.pages) {
+      fprintf(stderr, "Error:  Object is not a page object!\n");
+      return 0;
+   }
+
+   /* Look for any other pages with the same name */
+   do {
+      changed = False;
+      for (p = 0; p < xobjs.pages; p++) {
+	 if (p == thispage) continue;
+         if (xobjs.pagelist[p]->pageobj != NULL) {
+	    if (!strcmp(xobjs.pagelist[p]->pageobj->name, thispageobj->name)) {
+	       /* append ":2" to name or update suffix to ensure uniqueness */
+	       if (clnptr == NULL)
+		  sprintf(thispageobj->name, "%s:2", thispageobj->name);
+	       else
+		  sprintf(clnptr + 1, "%d", n + 1);
+	       changed = True;
+	       update = True;
+	       break;
+	    }
+	 }
+      }
+   } while (changed);
+   if (update) {
+      renamepage(thispage);
+      return -1;
+   }
+   return 0;
+}
+
 /*--------------------------------------*/
 /* Find connectivity of a selection	*/
 /*--------------------------------------*/
@@ -681,7 +740,7 @@ void swapschem(Widget w, u_int mode, caddr_t calldata)
 	       NEW_LABEL(pinlab, objectdata);
 	       (*pinlab)->pin = lpin->pin;
 	       (*pinlab)->color = lpin->color;
-	       (*pinlab)->rotation = 1;
+	       (*pinlab)->rotation = 0;
 	       (*pinlab)->scale = 1.0;
 	       (*pinlab)->justify = areastruct.justify; 
 	       (*pinlab)->position.x = 0;
@@ -748,7 +807,7 @@ void dobeforeswap(Widget w, caddr_t clientdata, caddr_t calldata)
       eventmode = NORMAL_MODE;
       popdata->dataptr = NULL;
       popdata->button = NULL; /* indicates that no button is assc'd w/ the popup */
-      popupprompt(w, "Enter name for new object:", "\0", makesymbol, popdata, False);
+      popupprompt(w, "Enter name for new object:", "\0", makesymbol, popdata, NULL);
    }
    else
       swapschem(w, (u_int)1, calldata);
@@ -815,6 +874,9 @@ void schemassoc(objectptr schemobj, objectptr symbolobj)
 
       /* Schematic takes the name of its associated symbol, by default */
       strcpy(schemobj->name, symbolobj->name);
+
+      /* Ensure that schematic (page) name is unique */
+      while (checkpagename(schemobj) < 0);
    
       setsymschem();	/* Set buttons and menu items appropriately */
    }
