@@ -48,8 +48,6 @@ extern short attachto;
 extern short textpos, textend;
 extern objectpair *pushlist;
 
-#define RADFAC 0.0174532925199   /* pi / 180 */
-
 /*----------------------------------------------------------------------*/
 /* Change filter to determine what types can be selected		*/
 /*----------------------------------------------------------------------*/
@@ -755,8 +753,14 @@ void selectarea()
 void startdesel(Widget w, caddr_t clientdata, caddr_t calldata)
 {
    if (eventmode == NORMAL_MODE) {
-      eventmode = DESEL_MODE;
-      Wprintf("Click on element to deselect");
+      if (areastruct.selects == 0)
+	 Wprintf("Nothing to deselect!");
+      else if (areastruct.selects == 1)
+	 objectdeselect();
+      else {
+         eventmode = DESEL_MODE;
+         Wprintf("Click on element to deselect");
+      }
    }
 }
 
