@@ -205,6 +205,10 @@ stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisi
    }
    if (dstr->type == TEXT_STRING) free(dstr->data.string);
    free(dstr);
+
+   /* attempt to merge, if legal */
+   mergestring(strptr);
+
    return strptr;
 }
 
@@ -881,10 +885,8 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
    if (drawlabel->pin) {
       if (!areastruct.schemon)
 	 goto enddraw;	/* Don't draw pins when schematic capture is off */
-      else {
+      else
 	 pinadjust(tmpjust, &(newpoint.x), &(newpoint.y), 1);
-         UDrawX(drawlabel);
-      }
    }
 #endif
 
@@ -1076,6 +1078,10 @@ enddraw:
 
    UPopCTM();
 
+#ifdef SCHEMA
+   if (drawlabel->pin && areastruct.schemon) UDrawXDown(drawlabel);
+#endif
+
    if ((defaultcolor != DOFORALL) && (passcolor != curcolor)) {
       XTopSetForeground(passcolor);
    }
@@ -1339,7 +1345,7 @@ void composefontlib(short cfont)
       *drawinst = (objinstptr) malloc(sizeof(objinst));
       (*drawinst)->type = OBJECT;
       (*drawinst)->color = DEFAULTCOLOR;
-      (*drawinst)->rotation = 1;
+      (*drawinst)->rotation = 0;
       (*drawinst)->scale = 1.0;
       (*drawinst)->thisobject = libobj;
       (*drawinst)->params = NULL;
