@@ -96,6 +96,7 @@ Widget	 message1, message2, message3, top, toolbar, overlay = NULL;
 Widget   wschema, wsymb, netbutton;
 #endif
 Display  *dpy;	             /* Works well to make this globally accessible */
+XtAppContext app;
 Window   win;
 Colormap cmap;
 short	 popups;	             /* total number of popup widgets on the screen */
@@ -170,10 +171,12 @@ static XtResource resources[] = {
       XtOffset(ApplicationDataPtr, selectpix), XtRString, "Gold3"},
   { "querycolor", "QueryColor", XtRPixel, sizeof(Pixel),
       XtOffset(ApplicationDataPtr, querypix), XtRString, "Turquoise"},
+  { "filtercolor", "FilterColor", XtRPixel, sizeof(Pixel),
+      XtOffset(ApplicationDataPtr, filterpix), XtRString, "SteelBlue3"},
   { "axescolor", "AxesColor", XtRPixel, sizeof(Pixel),
       XtOffset(ApplicationDataPtr, axespix), XtRString, "Antique White"},
   { "offbuttoncolor", "OffButtonColor", XtRPixel, sizeof(Pixel),
-      XtOffset(ApplicationDataPtr, buttonpix), XtRString, "Gray70"},
+      XtOffset(ApplicationDataPtr, buttonpix), XtRString, "Gray85"},
   { "auxiliarycolor", "AuxiliaryColor", XtRPixel, sizeof(Pixel),
       XtOffset(ApplicationDataPtr, auxpix), XtRString, "Green3"},
   { "barcolor", "BarColor", XtRPixel, sizeof(Pixel),
@@ -195,10 +198,12 @@ static XtResource resources[] = {
       XtOffset(ApplicationDataPtr, selectpix2), XtRString, "Gold"},
   { "querycolor2", "QueryColor", XtRPixel, sizeof(Pixel),
       XtOffset(ApplicationDataPtr, querypix2), XtRString, "Turquoise"},
+  { "filtercolor2", "FilterColor", XtRPixel, sizeof(Pixel),
+      XtOffset(ApplicationDataPtr, filterpix2), XtRString, "SteelBlue1"},
   { "axescolor2", "AxesColor", XtRPixel, sizeof(Pixel),
       XtOffset(ApplicationDataPtr, axespix2), XtRString, "NavajoWhite4"},
   { "offbuttoncolor2", "OffButtonColor", XtRPixel, sizeof(Pixel),
-      XtOffset(ApplicationDataPtr, buttonpix2), XtRString, "Gray70"},
+      XtOffset(ApplicationDataPtr, buttonpix2), XtRString, "Gray50"},
   { "auxiliarycolor2", "AuxiliaryColor", XtRPixel, sizeof(Pixel),
       XtOffset(ApplicationDataPtr, auxpix2), XtRString, "Green"},
   { "barcolor2", "BarColor", XtRPixel, sizeof(Pixel),
@@ -425,8 +430,8 @@ void resizetoolbar()
    Arg	wargs[4];
    Widget bwptr, lbwptr, bmax, lbmax;
    Dimension t_height, bheight, bwidth;
-   Boolean is_top = True;
-   int pybuttons = 0;
+   Widget *tool_list = NULL;
+   int pytools = 0;
 
    if (!XtIsRealized(toolbar)) return;
 
@@ -441,18 +446,18 @@ void resizetoolbar()
    XtGetValues(XtParent(toolbar), wargs, n); n = 0;
 
 #ifdef HAVE_PYTHON
-   pybuttons = pytoolbuttons();
+   tool_list = pytoolbuttons(&pytools);
 #endif
 
    /* Realign the tool buttons inside the fixed space */
 
-   for (i = 0; i < toolbuttons + pybuttons; i++) {
+   for (i = 0; i < toolbuttons + pytools; i++) {
 #ifdef HAVE_PYTHON
       if (i >= toolbuttons)
-	 bwptr = pytoolbutton(i - toolbuttons);
+	 bwptr = tool_list[i - toolbuttons];
       else
 #endif
-      bwptr = XtNameToWidget(toolbar, ToolBar[i].name);
+         bwptr = XtNameToWidget(toolbar, ToolBar[i].name);
       if (bwptr == (Widget)NULL) break;
 
       XtnSetArg(XtNheight, &bheight);
@@ -461,7 +466,6 @@ void resizetoolbar()
       bheight += (TBBORDER << 1);
       max_height += bheight;
       if (max_height > t_height) {
-	 is_top = True;
 	 bcol++;
          lbmax = bmax;
 	 tot_width += max_width;
@@ -485,6 +489,10 @@ void resizetoolbar()
    /* Reinstate callbacks */
    XtAddCallback(areastruct.area, XtNresize, (XtCallbackProc)resizetoolbar, NULL);
    XtAddCallback(areastruct.area, XtNresize, (XtCallbackProc)resizearea, NULL);
+
+#ifdef HAVE_PYTHON
+   if (tool_list != NULL) free(tool_list);
+#endif
 }
 
 #endif
@@ -688,7 +696,7 @@ void quitcheck(Widget w, caddr_t clientdata, caddr_t calldata)
       getgeneric(savebutton, w, quitcheck, NULL);
       promptstr = (char *)realloc(promptstr, strlen(promptstr) + 14);
       strcat(promptstr, "\nQuit anyway?");
-      popupprompt(w, promptstr, NULL, quit, savebutton, False);
+      popupprompt(w, promptstr, NULL, quit, savebutton, NULL);
       free(promptstr);
    }
    else {
@@ -761,6 +769,7 @@ void destroypopup(Widget button, popupstruct *callstruct, caddr_t calldata)
    /* free the allocated structure space */
 
    free(callstruct->buttonptr);
+   if (callstruct->filter != NULL) free(callstruct->filter);
    free(callstruct);
 
    /* in the case of "quitcheck", we want to make sure that the signal	*/
@@ -806,10 +815,9 @@ void xcgettext(Widget button, popupstruct *callstruct, caddr_t calldata)
       /* functions which use the file selector should look for directory name */
       /* in string rather than a file name, and take appropriate action.      */
 
-      if (callstruct->fileprompt == True) {
+      if (callstruct->filew != NULL) {
          if (lookdirectory(_STR2)) {
-            newfilelist(XtNameToWidget(XtParent(button), "Filelist"),
-                  callstruct->textw);
+	    newfilelist(callstruct->filew, callstruct);
 	    return;
          }
       }
@@ -827,8 +835,9 @@ void xcgettext(Widget button, popupstruct *callstruct, caddr_t calldata)
    (*(callstruct->setvalue))(callstruct->buttonptr->button,
 	 callstruct->buttonptr->dataptr);
 
-   if (callstruct->fileprompt == True)
-      newfilelist(XtNameToWidget(XtParent(button), "Filelist"), callstruct->textw);
+   if (callstruct->filew != NULL)
+      newfilelist(callstruct->filew, callstruct);
+
    destroypopup(button, callstruct, calldata);
 }
 
@@ -970,7 +979,7 @@ void updatename(Widget button, WidgetList callstruct, caddr_t calldata)
 /*-------------------------------------------------------------------------*/
 
 void popupprompt(Widget button, char *request, char *current, void (*function)(),
-	buttonsave *datastruct, Boolean fileprompt)
+	buttonsave *datastruct, const char *filter)
 {
     Arg         wargs[9];
     Widget      popup, dialog, okbutton, cancelbutton, entertext;
@@ -983,7 +992,7 @@ void popupprompt(Widget button, char *request, char *current, void (*function)()
     popupstruct	*okaystruct;
 
     height = (current == NULL) ? ROWHEIGHT * 4 : ROWHEIGHT * 5;
-    if (fileprompt) height += LISTHEIGHT;
+    if (filter) height += LISTHEIGHT;
 
     width = XTextWidth(appdata.xcfont, request, strlen(request)) + 20;
     bwidth = XTextWidth(appdata.xcfont, "Cancel", strlen("Cancel")) + 50;
@@ -1009,7 +1018,7 @@ void popupprompt(Widget button, char *request, char *current, void (*function)()
         popup, wargs, n); n = 0;
 
     XtnSetArg(XtNx, 20);
-    XtnSetArg(XtNy, ROWHEIGHT - 10 + (fileprompt ? LISTHEIGHT : 0));
+    XtnSetArg(XtNy, ROWHEIGHT - 10 + (filter ? LISTHEIGHT : 0));
     XtnSetArg(XtNstring, request);
     XtnSetArg(XtNborderWidth, 0);
     XtnSetArg(XtNgravity, WestGravity);
@@ -1027,9 +1036,10 @@ void popupprompt(Widget button, char *request, char *current, void (*function)()
     okaystruct = (popupstruct *) malloc(sizeof(popupstruct));
     okaystruct->buttonptr = datastruct;
     okaystruct->popup = popup;
-    okaystruct->fileprompt = fileprompt;
+    okaystruct->filter = (filter == NULL) ? NULL : strdup(filter);
     okaystruct->setvalue = function;
     okaystruct->textw = NULL;
+    okaystruct->filew = NULL;
 
     XtnSetArg(XtNx, width - bwidth - 20);
     XtnSetArg(XtNy, height - ROWHEIGHT - 10);
@@ -1045,7 +1055,7 @@ void popupprompt(Widget button, char *request, char *current, void (*function)()
        short		posit;
 
        XtnSetArg(XtNx, 20);
-       XtnSetArg(XtNy, ROWHEIGHT + 10 + (fileprompt ? LISTHEIGHT : 0));
+       XtnSetArg(XtNy, ROWHEIGHT + 10 + (filter ? LISTHEIGHT : 0));
        XtnSetArg(XtNheight, ROWHEIGHT + 5);
        XtnSetArg(XtNwidth, width - 40);
        XtnSetArg(XtNstring, current);
@@ -1070,7 +1080,7 @@ void popupprompt(Widget button, char *request, char *current, void (*function)()
 
        /* Generate file prompting widget */
 
-       if (fileprompt) genfilelist(dialog, entertext, width);
+       if (filter) genfilelist(dialog, okaystruct, width);
     }
     XtAddCallback(okbutton, XtNselect, (XtCallbackProc)xcgettext, okaystruct);
 
@@ -1409,6 +1419,7 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
    donestruct = (popupstruct *) malloc(sizeof(popupstruct));
    donestruct->popup = popup;
    donestruct->buttonptr = savebutton;
+   donestruct->filter = NULL;
    XtAddCallback(okbutton, XtNselect, (XtCallbackProc)destroypopup, donestruct);
 
    /* Send setfile() the widget entertext[0] in case because user sometimes
@@ -1496,7 +1507,7 @@ void Wprintf(char *string)
    XtSetValues(message3, wargs, 1);
 
    /* 10 second timeout */
-   printtime_id = XtAddTimeOut(10000, (XtTimerCallbackProc)clrmessage, NULL);
+   printtime_id = XtAppAddTimeOut(app, 10000, (XtTimerCallbackProc)clrmessage, NULL);
 }   
 
 /*------------------------------------------------------------------------------*/
@@ -1652,7 +1663,8 @@ void docommand()
 {
    Arg wargs[12];
    int n = 0;
-   int w, x, y;
+   Dimension w;
+   Position x, y;
    static char defaultTranslations[] = "<Key>Return:   execute()";
 
    if (overlay == NULL) {
@@ -1978,6 +1990,16 @@ int main(int argc, char *argv[])
    short initargc = argc;	/* because XtInitialize() absorbs the     */
    				/* -schem flag and renumbers argc! (bug?) */
 
+   char *argfb[] = {			/* Fallback resources */
+	"xcircuit*foreground : brown4", /* These are the values that 	*/
+	"xcircuit*background : beige",  /* not or cannot be explicitly	*/
+	"xcircuit.foreground : black",	/* initialized by		*/
+	"xcircuit.background : white",  /* XtGetApplicationResources()	*/
+	"xcircuit*borderWidth : 2",	/* below.			*/
+	"xcircuit*borderColor : Red",
+	NULL				/* Sentinel */
+   };
+
    /*-----------------------------------------------------------*/
    /* Find the root of the command called from the command line */
    /*-----------------------------------------------------------*/
@@ -2076,6 +2098,8 @@ int main(int argc, char *argv[])
    areastruct.center = True;
    areastruct.bboxon = False;
    areastruct.filter = SEL_ANY;
+   areastruct.filefilter = True;
+   areastruct.editinplace = True;
    areastruct.selects = 0;
    areastruct.selectlist = NULL;
    areastruct.manhatn = False;
@@ -2101,7 +2125,7 @@ int main(int argc, char *argv[])
 
    areastruct.topobject = (objinstptr) malloc(sizeof(objinst));
    areastruct.topobject->scale = 1.0;
-   areastruct.topobject->rotation = 1;
+   areastruct.topobject->rotation = 0;
    areastruct.topobject->position.x = 0;
    areastruct.topobject->position.y = 0;
    areastruct.topobject->params = NULL;
@@ -2138,7 +2162,8 @@ int main(int argc, char *argv[])
    /* Create the widget hierarchy */
    /*-----------------------------*/
 
-   top = XtInitialize(argv0, "XCircuit", NULL, 0, &argc, argv);
+   top = XtOpenApplication(&app, "XCircuit", NULL, 0, &argc, argv,
+		argfb, applicationShellWidgetClass, NULL, 0);
    dpy = XtDisplay(top);
    win = DefaultRootWindow(dpy);
    cmap = DefaultColormap(dpy, DefaultScreen(dpy));
@@ -2162,7 +2187,7 @@ int main(int argc, char *argv[])
    /* Get the application resources */
    /*-------------------------------*/
 
-   XtAddConverter(XtRString, XtRPixel, (XtConverter)CvtStringToPixel, NULL, 0);
+   XtAppAddConverter(app, XtRString, XtRPixel, (XtConverter)CvtStringToPixel, NULL, 0);
    XtGetApplicationResources(top, &appdata, resources, XtNumber(resources), 
 	NULL, 0);
 
@@ -2591,14 +2616,14 @@ int main(int argc, char *argv[])
    /*---------------------------------------------------*/
 
    xobjs.save_interval = appdata.timeout;
-   xobjs.timeout_id = XtAddTimeOut((u_long)60000 * xobjs.save_interval,
+   xobjs.timeout_id = XtAppAddTimeOut(app, (u_long)60000 * xobjs.save_interval,
 	(XtTimerCallbackProc)savetemp, NULL);
 
    /*----------------------*/
    /* Start the event loop */
    /*----------------------*/
 
-   XtMainLoop();
+   XtAppMainLoop(app);
    return EXIT_SUCCESS;
 }
 
