@@ -17,7 +17,6 @@
 #define max(a,b)		((a) < (b) ? (b) : (a))
 #define min(a,b)		((a) < (b) ? (a) : (b))
 #define Number(a)		(void *)((u_int) a)
-#define ROT_UNIT (360 / RSTEPS) /* degrees per xcircuit rotational unit */
 
 /*----------------------------------------------------------------------*/
 /* Lengthier define constructs						*/
@@ -125,7 +124,7 @@ typedef struct {
 #define SCALEFAC  1.5 /* zoom in/out scaling multiplier 		*/
 #define SUBSCALE 0.67 /* ratio of subscript size to normal script size 	*/
 #define SBARSIZE   13 /* Pixel size of the scrollbar 			*/
-#define RSTEPS     72 /* Number of allowed rotation positions 		*/
+#define RSTEPS     72 /* Number of points defining a circle approx.	*/
 #define SPLINESEGS 20 /* Number of points per spline approximation 	*/
 #define DELBUFSIZE 10 /* Number of delete events to save for undeleting */
 #define MINAUTOSCALE 0.75 /* Won't automatically scale closer than this */
@@ -157,6 +156,7 @@ typedef struct {
 
 /*----------------------------------------------------------------------*/
 
+#define RADFAC 0.0174532925199   /* pi / 180 */
 #define INVRFAC 57.295779       /* 180 / pi */
 
 /*----------------------------------------------------------------------*/
@@ -196,14 +196,16 @@ UIPARAM_MODE
 #define RECOVER    4
 
 /*----------------------------------------------------------------------*/
-/* Text justification styles						*/
+/* Text justification styles and other parameters (bitmask)		*/
 /*----------------------------------------------------------------------*/
 
 #define NOTLEFT		1
 #define RIGHT		2
 #define NOTBOTTOM	4
 #define TOP		8
-#define FLIPINV		16
+#define FLIPINV		16	/* 1 if text is flip-invariant */
+#define PINVISIBLE	32	/* 1 if pin visible outside of object */
+#define PINNOOFFSET	64	/* 0 if pin label offset from position */
 
 /*----------------------------------------------------------------------*/
 /* Text string part: types						*/
@@ -632,6 +634,10 @@ typedef struct _xcobject {
 
 /*----------------------------------------------------------------------*/
 /* Transformation matrices						*/
+/* Works like this (see also PostScript reference manual):		*/
+/*   [x' y' 1] = [x y 1] * | a  d  0 |					*/
+/*			   | b  e  0 |					*/
+/*			   | c  f  1 |					*/
 /*----------------------------------------------------------------------*/
 
 typedef struct _matrix *Matrixptr;
@@ -665,11 +671,14 @@ typedef struct {
 } buttonsave;
 
 typedef struct {
-   Widget	popup;
-   Widget	textw;
-   void		(*setvalue)();
-   buttonsave	*buttonptr;
-   Boolean	fileprompt;
+   Widget	popup;		/* Popup widget		*/
+   Widget	textw;		/* Text entry widget	*/
+   Widget	filew;		/* File list window 	*/
+   void		(*setvalue)();	/* Callback function	*/
+   buttonsave	*buttonptr;	/* Button widget calling popup	     */
+   char		*filter;	/* Extension filter for highlighting */
+				/*  files. NULL for no file window.  */
+				/*  NULL-string for no highlights.   */
 } popupstruct;
 
 typedef struct {
@@ -679,6 +688,13 @@ typedef struct {
    void		*dataptr;
 } propstruct;
 
+typedef struct {
+   char		*filename;
+   int		filetype;
+} fileliststruct;
+
+enum {DIRECTORY = 0, MATCH, NONMATCH};	/* file types */
+
 /*----------------------------------------------------------------------*/
 /* Initial Resource Management						*/
 /*----------------------------------------------------------------------*/
@@ -687,14 +703,14 @@ typedef struct {
    /* color scheme 1 */
    Pixel	fg, bg;
    Pixel	gridpix, snappix, selectpix, axespix;
-   Pixel	buttonpix, querypix;
+   Pixel	buttonpix, querypix, filterpix;
    Pixel	auxpix, barpix, bboxpix;
 
    /* color scheme 2 */
 
    Pixel	fg2, bg2;
    Pixel	gridpix2, snappix2, selectpix2, axespix2;
-   Pixel	buttonpix2, querypix2;
+   Pixel	buttonpix2, querypix2, filterpix2;
    Pixel	auxpix2, barpix2, bboxpix2;
 
    int		width, height, timeout;
@@ -863,6 +879,8 @@ typedef struct {
    Boolean	axeson;
    Boolean	invert;
    Boolean	mapped;		/* indicates if window is drawable */
+   Boolean	filefilter;	/* Is the file list filtered? */
+   Boolean	editinplace;
 #ifdef SCHEMA
    Boolean	schemon;
    Boolean	pinpointon;
